from .movie_review import MovieReview
from .customer_review import CustomerReview
from .imdb import IMDB
from .multi_nli import MultiNLI
from .news_group20 import NewsGroup20
from .reuters_news import ReutersNews
from .ptb import PTB
from .wikitext2 import WikiText2
from .wikitext103 import WikiText103
from .text8 import Text8
